import { createSlice, PayloadAction } from '@reduxjs/toolkit';
import type { RootState } from './store';
import { VisualizationConfiguration } from '../../vis/common';
import { isEqual } from 'lodash-es';

export type VisStateSettings = {
  [id: string]: VisualizationConfiguration;
};
export type VisState = {
  active?: string;
  visualizations: VisStateSettings;
};

export const initialState: VisState = {
  // active: 'NodeLinkVis',
  visualizations: {},
};

export const visualizationSlice = createSlice({
  name: 'visualization',
  initialState,
  reducers: {
    removeVisualization: (state, action: PayloadAction<string>) => {
      if (state.visualizations[action.payload]) {
        delete state.visualizations[action.payload];
      }
    },
    setActiveVisualization: (state, action: PayloadAction<string>) => {
      state.active = action.payload;
    },
    setVisualizationState: (state, action: PayloadAction<VisState>) => {
      if (action.payload.active && !isEqual(action.payload, state)) {
        state.active = action.payload.active;
        state.visualizations = action.payload.visualizations;
      }
    },
    updateVisualization: (state, action: PayloadAction<{ id: string; settings: any }>) => {
      const { id, settings } = action.payload;
      state.visualizations[id] = settings;
    },
    reorderVisState: (state, action: PayloadAction<{ [id: string]: VisualizationConfiguration }>) => {
      state.visualizations = action.payload;
    },
  },
});

export const { removeVisualization, setActiveVisualization, setVisualizationState, updateVisualization, reorderVisState } =
  visualizationSlice.actions;

export const visualizationState = (state: RootState) => state.visualize;
export const visualizationAllSettings = (state: RootState) => state.visualize.visualizations;

export default visualizationSlice.reducer;
